package com.gic.plug.web.utils;

import com.gic.store.dto.StoreWidgetLogDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.plug.web.qo.StoreWidgetQO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.dto.StoreWidgetLogSaveDTO;
import com.gic.store.service.StoreWidgetApiService;
import com.gic.store.service.StoreWidgetLogApiService;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 门店选择器操作日志工具类
 * @ClassName:
 * @Description:  
 * @author guojuxing 
 * @date 2021/1/13 7:54 PM 
 */
@Component
public class StoreWidgetLogUtils {
    private static Logger logger = LogManager.getLogger(StoreWidgetLogUtils.class);
    @Autowired
    private StoreWidgetLogApiService storeWidgetLogApiService;
    @Autowired
    private StoreWidgetApiService storeWidgetApiService;

    /**
     * 写门店选择器日志
     * @param storeWidgetQO
     * @param newStoreWidgetId
     */
    public void saveStoreWidgetLog(StoreWidgetQO storeWidgetQO, Integer newStoreWidgetId) {
        Integer storeWidgetId = storeWidgetQO.getStoreWidgetId();
        //如果是编辑，则会有值，用于判断创建还是编辑
        StoreWidgetDTO oldStoreWidget = getOldStoreWidget(storeWidgetId);
        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO;

        if (oldStoreWidget == null) {
            logSaveDTO = buildStoreWidgetLogWhenCreate(storeWidgetQO, newStoreWidgetId);
        } else {
            //copy旧门店选择器ID上挂载的日志到新的ID上面
            copyStoreWidgetLog(storeWidgetId, newStoreWidgetId);
            //新门店选择器ID上新增新的日志
            logSaveDTO = buildStoreWidgetLogWhenUpdate(oldStoreWidget, storeWidgetQO, newStoreWidgetId);
        }
        //写日志
        storeWidgetLogApiService.saveStoreWidgetLog(logSaveDTO);
    }

    /**
     * 因为该字段和其他数据分离开来，需要根据ID查询旧记录数据
     * @param storeWidgetId
     * @param authSearchParam
     * @return
     */
    public String getAuthSearchParamData(Integer storeWidgetId, String authSearchParam) {
        if (StringUtils.isBlank(authSearchParam)) {
            StoreWidgetDTO oldStoreWidget = getOldStoreWidget(storeWidgetId);
            if (oldStoreWidget != null) {
                return oldStoreWidget.getAuthSearchParam();
            }
        }
        return authSearchParam;
    }

    /**
     * 只更新权限范围数据的日志
     * @param storeWidgetId
     * @param authSearchParam
     */
    public void saveStoreWidgetWhenEditAuth(Integer storeWidgetId, String authSearchParam) {
        StoreWidgetDTO oldStoreWidget = getOldStoreWidget(storeWidgetId);
        if (oldStoreWidget != null) {
            //操作日志对象组装
            StoreWidgetQO storeWidgetQO = new StoreWidgetQO();
            storeWidgetQO.setAuthSearchParam(authSearchParam);
            StoreWidgetLogSaveDTO logSaveDTO = buildStoreWidgetLogWhenUpdate(oldStoreWidget, storeWidgetQO, storeWidgetId);
            //写日志
            storeWidgetLogApiService.saveStoreWidgetLog(logSaveDTO);
        }
    }

    /**
     * copy旧门店选择器ID上挂载的日志到新的ID上面
     * @param oldStoreWidgetId
     * @param newStoreWidgetId
     */
    private void copyStoreWidgetLog(Integer oldStoreWidgetId, Integer newStoreWidgetId) {
        //查询旧uuid上面挂载的日志数据
        ServiceResponse<List<StoreWidgetLogDTO>> response = storeWidgetLogApiService.listStoreWidgetLog(oldStoreWidgetId);
        if (response.isSuccess()) {
            List<StoreWidgetLogDTO> list = response.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                //复制数据到新uuid上面
                List<StoreWidgetLogDTO> copyList = list.stream().map(e ->
                        e.setStoreWidgetId(newStoreWidgetId)
                                .setStoreWidgetLogId(null)).collect(Collectors.toList());
                //
                storeWidgetLogApiService.tmpInsert(copyList);
            }
        }
    }


    private StoreWidgetDTO getOldStoreWidget(Integer storeWidgetId) {
        if (storeWidgetId == null) {
            return null;
        }
        ServiceResponse<StoreWidgetDTO> oldStoreWidgetResponse = storeWidgetApiService.getStoreWidget(storeWidgetId);
        if (!oldStoreWidgetResponse.isSuccess()) {
            logger.info("根据门店选择器ID:{}，查询不到记录", storeWidgetId);
            return null;
        }
        return oldStoreWidgetResponse.getResult();
    }

    /**
     * 组装日志对象-编辑场景
     * @param oldStoreWidget 老数据
     * @param storeWidgetQO 前端传入的参数
     * @param newStoreWidgetId 新增门店选择器产生的ID
     * @return
     */
    private static StoreWidgetLogSaveDTO buildStoreWidgetLogWhenUpdate(StoreWidgetDTO oldStoreWidget, StoreWidgetQO storeWidgetQO,
                                                                      Integer newStoreWidgetId) {
        String authSearchParam = storeWidgetQO.getAuthSearchParam();
        String searchParam = storeWidgetQO.getSearchParam();

        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO = buildCommon(newStoreWidgetId);

        //编辑
        //编辑类型
        logSaveDTO.setOperationType(2);
        //是否是权限变更范围类型
        boolean isNotAuthChangeType = StringUtils.isBlank(authSearchParam) || authSearchParam.equals(oldStoreWidget.getAuthSearchParam());
        if (isNotAuthChangeType) {
            //编辑筛选条件类型
            logSaveDTO.setLogType(2);

            logSaveDTO.setSearchParamBefore(oldStoreWidget.getSearchParam());
            if (StringUtils.isNotBlank(searchParam)) {
                logSaveDTO.setSearchParamAfter(searchParam);
            }
        } else {
            //权限变更范围类型
            logSaveDTO.setLogType(1);
            //记录变更前的权限范围数据
            logSaveDTO.setSearchParamBefore(oldStoreWidget.getAuthSearchParam());
            if (StringUtils.isNotBlank(authSearchParam)) {
                //记录变更后的权限范围数据
                logSaveDTO.setSearchParamAfter(authSearchParam);
            }
        }
        return logSaveDTO;
    }

    /**
     * 组装日志对象-创建场景
     * @param storeWidgetQO 前端传入的参数
     * @param newStoreWidgetId 新增门店选择器产生的ID
     * @return
     */
    private static StoreWidgetLogSaveDTO buildStoreWidgetLogWhenCreate(StoreWidgetQO storeWidgetQO, Integer newStoreWidgetId) {
        String authSearchParam = storeWidgetQO.getAuthSearchParam();
        String searchParam = storeWidgetQO.getSearchParam();

        //操作日志对象组装
        StoreWidgetLogSaveDTO logSaveDTO = buildCommon(newStoreWidgetId);

        //创建
        logSaveDTO.setOperationType(1);
        boolean isAuthChangeType = StringUtils.isNotBlank(authSearchParam);
        if (isAuthChangeType) {
            //权限变更范围类型
            logSaveDTO.setLogType(1);
            logSaveDTO.setSearchParamAfter(authSearchParam);
        } else {
            //编辑筛选条件类型
            logSaveDTO.setLogType(2);
            logSaveDTO.setSearchParamAfter(searchParam);
        }
        return logSaveDTO;
    }

    private static StoreWidgetLogSaveDTO buildCommon(Integer newStoreWidgetId) {
        StoreWidgetLogSaveDTO logSaveDTO = new StoreWidgetLogSaveDTO();
        logSaveDTO.setOperationId(UserDetailUtils.getUserDetail().getUserId());
        logSaveDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        logSaveDTO.setStoreWidgetId(newStoreWidgetId);
        return logSaveDTO;
    }
}
